# Sensitivity and inspiral range of KAGRA (and KAGRA+)
# Originally converted from kagra_sensitivity.m into python script, and modified for parameter search
# for-loops omitted from matlab version to speed up, keeping results the same as kagra_sesitivity.m

from math import *
import numpy as np
import matplotlib.pyplot as plt
import scipy.integrate as integ
i=1j

#####physical constants and other basic parameters
#from Rev. Mod. Phys. 88, 035009 (2016)
c=2.99792458e8        #speed of light (m/s)
hbar=1.054571800e-34  #Dirac constant (J*s)
kb=1.38064852e-23     #Boltzmann constant (J/K)
e=1.6021766208e-19    #electric charge (C)
G=6.67384e-11         #gravitational constant (m**3/kg/s**2)
g=9.80665             #gravitational acceleration (m/s**2)
#see http://asa.usno.navy.mil/SecK/Constants.html
msolar=1.9884e30                               #mass of the sun (kg)
Gpc=3.085677581491367278913937957796471611e25  #1Gpc (m)
Mpc=Gpc/1e3  #1Mpc (m)

lamb=1.064e-6         #laser wave length (m)
omegal=2*pi*c/lamb    #laser frequency (rad/s)
L=3e3     #arm length (m)

fres=1000               #frequency resolution
f=np.logspace(0,4,fres)
omega=2*pi*f
# Frequency resolution could be insufficient for violin modes. Change the resolution if necessary.

def sensitivity(phidet=pi/180*86.5,xi=pi/180*135.1,rSRM=0.92,Tm=22,I0attenuation=1,l3=35e-2,wsafe=12.5765,mass=22.8,LF=0,SQZ=0,FC=0,CoatImprove=0):
	# LF: LF option, SQZ: squeezing level, FC: filter cavity length, CoatImprove: coating improvement factor
    ## DEFINE PARAMETERS ##########
    #RSE configuration
    
    #RSE='DRSE'     # 'BRSE', 'BRSEhomo', or 'DRSE' (BRSEhomo for optimized homodyne readout phase)
    
    #####interferometer parameters
    
    #Detune phase and homodyne phase are optimized based on BNS range
    #Re-optimization is necessary if other parameters are changed

#	if RSE=='DRSE':     #detuned RSE
#    	#phidet=pi/180*86.5   #detune phase
#      	#xi=pi/180*135.1      #homodyne phase
#      	pass
#	elif RSE=='BRSEhomo':    #broadband RSE with homodyne
#    	phidet=pi/180*90
#     	xi=pi/180*119.1
#	elif RSE=='BRSE':    #broadband RSE without homodyne
#      	phidet=pi/180*90
#      	xi=pi/180*90

    trans=0.004          #power transmittivity of ITM
    #rSRM=0.92            #amplitude reflectivity of SRM
    roundtriploss=100e-6 #round trip loss of arm
    lambSR=0.002       #power loss at SRM
    lambPD=0.1         #power loss at PD
    Sadd=0*(pi**2/8-1)
    
    #####mirror parameters
    
    rhom=4.0e3      #density of mirror (kg/m**3) #sapphire
    radius=11e-2*(mass/22.8)**(1./3)    #radius of mirror (m); scale size with respect to mass (fixed aspect ratio)
    height=15e-2*(mass/22.8)**(1./3)    #thickness of mirror (m); scale size with respect to mass (fixed aspect ratio)
    #mass=22.8       #mirror mass (kg)
    w1=3.5e-2*(mass/22.8)**(1./3)       #beam radius at ITM (m); scale size with respect to mass
    w2=3.5e-2*(mass/22.8)**(1./3)       #beam radius at ETM (m); scale size with respect to mass
    #Tm=22           #temperature at TM [k]
    E0=400e9        #Youngs modulus of mirror (Pa), see Phys. Lett. A 264, 1 (1999)
    sigma=0.29      #poisson ratio of mirror, see Phys. Lett. A 264, 1 (1999)
    Qm=1e8          #Q-value of mirror
    
    
    #From Uchiyama-san. Origninally from Touloukian and Ho, Thermophysical Properties of Matter: The TRPC Data Series, Plenum Press (1970).
    #see PRL 108, 141101 (2012).
    kappa20=(15700./15880)*((5*Tm**2.75)**(-4./5)+(10**10.25*Tm**(-3.8))**(-4./5))**(-5./4) #thermal conductivity
    Cth20=(0.69/0.80)*Tm**3.14/3.8/rhom  #specific heat per unit mass
    alpha20=(5.6/5.496)*Tm**2.99/10**12.15  #thermal linear expansion
    
    
    #####coating parameters
    
    #number of coating layers
    Ncoa1=11       #ITM
    Ncoa2=20       #ETM
    
    #Youngs modulus (Pa)
    Ys=73.2e9       #silica, see PRD 93, 012007 (2016)
    Yt=140e9        #tantala, see PRD 70, 082003 (2004)
    
    #poisson ratio
    sigmas=0.164    #silica, see PRD 93, 012007 (2016)
    sigmat=0.23     #tantala, see PRD 70, 082003 (2004)
    
    #refractive index
    n1=1.442        #silica, see PRD 93, 012007 (2016)
    n2=2.035        #tantala, see CQG 27, 084030 (2010)
    
    #thickness of layers (m)
    dcoa1s=(Ncoa1+1)*lamb/(4*n1)  #silica, ITM
    dcoa1t=Ncoa1*lamb/(4*n2)      #tantala, ITM
    dcoa2s=(Ncoa2+1)*lamb/(4*n1)  #silica, ETM
    dcoa2t=Ncoa2*lamb/(4*n2)      #tantala, ITM
    
    #loss angle
    phic1=3e-4       #silica coating
    phic2=5e-4       #tantala coating
    if CoatImprove:
        phic1/=CoatImprove
        phic2/=CoatImprove
    
    #####suspension parameters
    
    rhos=4.0e3       #density of suspension wire (kg/m**3) #sapphire
    
    #mass (kg)
    m1=20.5          #IM
    if LF:
        m1=20.5*4		# for KN KAGRA-LF
    m2=0.055*4       #blade spring
    m3=mass          #TM
    
    #resonant frequency of blade spring (Hz)
    wh2=2*pi*2e3     #horizontal
    fmg=14.5         #vertical
    
    #radius of wire (m)
    r1=0.6e-3/2     #wire1 between IM and MN; scale with IM mass
    r2=1.6e-3/2*(mass/22.8)**(1./2)*(wsafe/12.5765)**(1./2)      #wire2 between blade spring and TM; scale with mirror mass (wsafe for safety factor; default value calculated with modulus of rupture to be 350 MPa (360e6*pi*r2**2/g*4/22.8))
    if LF:
        r1=0.2e-3/2 # for KN KAGRA-LF
        #r2=3.2e-4/2 # for KN KAGRA-LF
    
    #Youngs modulus (Pa)
    Y1=1.3e11        #wire1 #CuBe
    Y2=4.0e11        #blade spring #sapphire
    Y3=4.0e11        #wire2 #sapphire
    
    #length of wire (m)
    l1=26.1e-2       #wire1
    if LF:
    	l1=26.1e-2*3       #wire1 for KN KAGRA-LF
    l2=0.5e-2        #blade spring
    #l3=35e-2         #wire2
    
    #loss angle of wire
    phi1=5e-6        #wire1
    phi2=7e-7        #blade spring
    phi3=2e-7        #wire2
    
    #temperature [k]
    T1=16            #wire1
    T2=16            #blade spring
    T3=(T2+Tm)/2     #wire2
    
    #loss angle normalized by T3
    phih1=phi1*T1/T3 #horizontal wire1
    phih2=phi2*T2/T3 #horizontal blade spring
    phih3=phi3       #horizontal wire2
    phiv1=phi1*T1/T3 #vertical wire1
    phiv2=phi2*T2/T3 #vertical blade spring
    phiv3=phi3       #vertical wire2
    
    #coupling from vertical mode to horizontal mode
    VHC=1./200
    
    #number of wires
    N1=4
    N2=4
    N3=4
    
    #number of bending points
    b1=2
    b2=2
    b3=2
    
    ## DETERMINATION OF LASER POWER AT BS ##########
    #Power at BS is determined from sapphre mirror temperature and np.absorption.
    
    subloss=50e-6*1e2       #energy loss rate of substrate (/m)
    #see JGW-G1706429-v1 (2017)
    coaloss=0.5e-6          #energy loss rate of coating
    outside=0.05            #radiation energy from outside (W)
    if LF:
	    outside=0.003	# for KN KAGRA-LF
    #see CQG 31, 224003 (2014)
    
    alphak=7.98*(r2/0.8e-3)	# included r2-dependance on March 29, 2018
    betak=2.2      #kappa=alphak*T**betak ,thermal conductivity (W/K/m)
    #see CQG 31, 105004 (2014)
    
    Kabs=N3*alphak/(betak+1)*pi*r2**2/l3*(Tm**(betak+1)-T2**(betak+1)) #total heatflow
    Pmich=(Kabs-outside)/(2*subloss*height+coaloss*4/trans) #power between BS and ITM
    I0=2*Pmich          #power at BS
    I0=I0*I0attenuation	#attenuated from maximum power
    if I0 < 0:
    	I0=1e-3	# make it very small if minus
    #print "Power at BS: %d W" % (I0)
    
    ## NOISE CALCULATIONS ##########
    #####seismic noise#####
    # fitting function of the seismic noise calculated by R. Takahashi.
    hseisorig=2*2*np.sqrt((4.1e-17*f**(-6.8))**2+(1.9e-16*f**(-8))**2+(1.8e-18*f**(-5.7))**2)
    
    # gravity gradient noise
    # fitting function of the GGN noise calculated by J. Harms (see CQG 29, 124007 (2012)).
    hGGN=1.2e-16/f**4/L
    
    hseis=np.sqrt(hseisorig**2+hGGN**2)
    
    
    #####suspension thermal noise#####
    #see VIR-015E-09 (2009)
    #violin mode parameters

    lsus=l3
    Ewire=Y3*(1+i*phi3)   #complex Youngs modulus (Pa)
    Icwire=pi*r2**4/4      #area moment of inertia (m**2)
    tension=mass*g/N3     #tension per wire (N)
    rholine=rhos*pi*r2**2  #linear density (kg/m)
    
    # wave number of elastic fibers (1/m)
    ks=np.sqrt((-tension+np.sqrt(tension**2+4*Ewire*Icwire*rholine*omega**2))/(2*Ewire*Icwire))
    
    # wave number associated with the flexural stiffness
    ke=np.sqrt((tension+np.sqrt(tension**2+4*Ewire*Icwire*rholine*omega**2))/(2*Ewire*Icwire))
    
    # effective spring constant (N/m)
    # Calculated by K. Somiya and confirmed by K. Komori. The spring constant Eq.(48) in PPPnote (VIR-015E-09) is the limitting value of ke*lsus -> infinity in our case.
    Ksusp=N3*Ewire*Icwire*ke*ks*(ke**2+ks**2)*(ke*np.cos(ks*lsus)*np.sinh(ke*lsus)+ks*np.sin(ks*lsus)*np.cosh(ke*lsus))/(2*ke*ks*(1-np.cos(ks*lsus)*np.cosh(ke*lsus))+(ke**2-ks**2)*np.sin(ks*lsus)*np.sinh(ke*lsus))
    
    #spring constant and resonant frequency
    Ecube=Y1*(1+i*phih1)          #complex Youngs modulus (Pa)
    I1=pi*r1**4/4                  #area moment of inertia (m**2)
    tensioncube=(m1+m2+m3)*g/N1   #tension per CuBe wire (N)
    
    kh1=N1*tensioncube/(l1-2*np.sqrt(Ecube*I1/tensioncube))      #hor. spring constatnt of wire1 (N/m)
    kh2=m2*wh2**2*(1+i*phih2)      #hor. spring constatnt of blade spring (N/m)
    kh3=Ksusp
    
    wv1=np.sqrt(pi*r1**2*Y1*N1/(l1*m1))  #ver. wire1 (Hz)
    kv1=m1*wv1**2*(1+i*phiv1)         #ver. spring constatnt of wire1 (N/m)
    
    wv2=2*pi*fmg*np.sqrt((m2+m3)/m2)         #ver. blade spring (Hz)
    kv2=(m2+m3)*(2*pi*fmg)**2*(1+i*phiv2)  #ver. spring constatnt of blade spring (N/m)
    
    wv3=np.sqrt(pi*r2**2*Y3*N3/(l3*m3))  #ver. wire2 (Hz)
    kv3=m3*wv3**2*(1+i*phiv3)         #ver. spring constatnt of wire2 (N/m)
    
    
    #matrix of horizontal spring constatnt (N/m)
    Dh11=-m1*omega**2+kh1+kh2
    Dh12=-kh2*np.ones(fres)
    Dh13=0*np.ones(fres)
    Dh21=Dh12
    Dh22=-m2*omega**2+kh2+kh3
    Dh23=-kh3
    Dh31=Dh13
    Dh32=Dh23
    Dh33=-m3*omega**2+kh3
    
    # matrix of vertical spring constatnt (N/m)
    Dv11=-m1*omega**2+kv1+kv2
    Dv12=-kv2*np.ones(fres)
    Dv13=0*np.ones(fres)
    Dv21=Dv12
    Dv22=-m2*omega**2+kv2+kv3
    Dv23=-kv3*np.ones(fres)
    Dv31=Dv13
    Dv32=Dv23
    Dv33=-m3*omega**2+kv3
    
    # for-loop omitted from matlab version for speed up (no change in results)
    #matrix of horizontal impedance (N/m/Hz) and inversion
    Zh=np.array([[Dh11,Dh12,Dh13],[Dh21,Dh22,Dh23],[Dh31,Dh32,Dh33]])/(i*omega)
    invZh=np.transpose(np.linalg.inv(np.transpose(Zh)))	# gives invZh[:,:,k]=np.linalg.inv(Zh[:,:,k])
    #matrix of vertical impedance (N/m/Hz) and inversion
    Zv=np.array([[Dv11,Dv12,Dv13],[Dv21,Dv22,Dv23],[Dv31,Dv32,Dv33]])/(i*omega)
    invZv=np.transpose(np.linalg.inv(np.transpose(Zv)))
    ###calculation of eq.(63) in PPPnote (VIR-015E-09)###
    # The fluctuation of TM is derived from 3*3 component of the inverse matrix of impedance.
    Fsqh=4*kb*T3*np.real(invZh)/omega**2
    Sthh=Fsqh[2,2,:]
    Fsqv=4*kb*T3*np.real(invZv)/omega**2
    Sthv=Fsqv[2,2,:]
    
    #strain of thermal noise with 4 TMs
    hsusph=np.sqrt(np.abs(Sthh))*2/L
    hsuspv=np.sqrt(np.abs(Sthv))*VHC*2/L
    hsusp=np.sqrt(hsusph**2+hsuspv**2)	# hsusp value slightly different from matlab code at 1e-11 level
    
    
    ######mirror thermal noise#####
    #strain of substrate thermal noise for 2 ITMs and ETMs
    #see CQG, 29, 124007 (2012)
    hmirsub=np.sqrt(2)/L*np.sqrt(4*kb*Tm*(1-sigma**2)/(np.sqrt(pi)*E0*Qm*omega)*(1/w1+1/w2))
    
    #thermoelastic noise calculations
    #see PRD, 82, 127101 (2010)
    #normalized frequency
    omegac1=omega*Cth20*rhom*w1**2/(2*kappa20)
    omegac2=omega*Cth20*rhom*w2**2/(2*kappa20)
    
    #calculation of Jhyper
    #It is confirmed by K. Komori that this integration is the same as the analytical calculation using error function.
    resolution=0.01
    up=10    #upper limit of integration instead of infinity
    bin=int(floor(up/resolution))
    u=np.linspace(0,up,bin+1)
    
    # for-loop omitted and simplified from matlab code for speed up (no change in result)
    uu,omec1=np.meshgrid(u,omegac1)
    uu,omec2=np.meshgrid(u,omegac2)
    
    integ1=u**2*np.exp(-u**2/2)*(1-u/np.sqrt(2)*np.sqrt((u**2+np.sqrt(u**4+omec1**2))/(u**4+omec1**2)))
    integ2=u**2*np.exp(-u**2/2)*(1-u/np.sqrt(2)*np.sqrt((u**2+np.sqrt(u**4+omec2**2))/(u**4+omec2**2)))
    
    integral1=np.sum(integ1[:,1:]+integ1[:,:-1],axis=1)/2*resolution # nulling changed from matlab code
    integral2=np.sum(integ2[:,1:]+integ2[:,:-1],axis=1)/2*resolution # nulling changed from matlab code
    
    Jhyper1int=np.sqrt(2/pi)*integral1/omegac1**2
    Jhyper2int=np.sqrt(2/pi)*integral2/omegac2**2
    
    #strain of thermoelastic noise for 2 ITMs and ETMs
    hmirelaint=np.sqrt(2)/L*np.sqrt(4*kb*Tm**2*alpha20**2*(1+sigma)**2/(np.sqrt(pi)*kappa20)*(w1*Jhyper1int+w2*Jhyper2int))
    
    #coating thermal noise calculations
    #see CQG, 19, 897 (2002)
    #thermal noise of silica coating
    hmircoas=np.sqrt(4*kb*Tm/(pi*E0**2*omega))*np.sqrt(phic1*(Ys**2*(1+sigma)**2*(1-2*sigma)**2+E0**2*(1+sigmas)**2*(1-2*sigmas))/(Ys*(1-sigmas**2))*(dcoa1s/w1**2+dcoa2s/w2**2))
    
    #thermal noise of tantala coating
    hmircoat=np.sqrt(4*kb*Tm/(pi*E0**2*omega))*np.sqrt(phic2*(Yt**2*(1+sigma)**2*(1-2*sigma)**2+E0**2*(1+sigmat)**2*(1-2*sigmat))/(Yt*(1-sigmat**2))*(dcoa1t/w1**2+dcoa2t/w2**2))
    
    #strain of coating thermal noise for 2 ITMs and ETMs
    hmircoa=np.sqrt(2)/L*np.sqrt(hmircoas**2+hmircoat**2)
    
    #total mirror thermal noise
    hmir=np.sqrt(hmirsub**2+hmirelaint**2+hmircoa**2)
    
    
    ######quantum noise#####
    #see PRD, 64, 042006 (2001)
    #parameters
    gamma=trans*c/(4*L)       #cavity pole (Hz)
    tSRM=np.sqrt(1-rSRM**2)       #SRM transmittance
    beta=np.arctan(omega/gamma)   #phase shift in the arm cavity
    
    #laser power quantum noise reaches the SQL at the cavity pole
    ISQL=mass*L**2*gamma**4/(4*omegal)
    
    #coefficient showing how much the mirror moves
    kappa=I0/ISQL*2*gamma**4/(omega**2*(gamma**2+omega**2))
    
    #loss parameter of arms defined in KLMTV paper
    epsilon=2*roundtriploss/trans
    
    #quantum noise calculations
    # strain of SQL
    hsql=np.sqrt(8*hbar/(mass*omega**2*L**2))
    
    #components of eq.(5.13) in PRD, 64, 042006 (2001)
    C11=np.sqrt(1-lambPD)*((1+rSRM**2)*(np.cos(2*phidet)+kappa*np.sin(2*phidet)/2)-2*rSRM*np.cos(2*beta)-epsilon/4*(-2*(1+np.exp(2*i*beta))**2*rSRM+4*(1+rSRM**2)*np.cos(beta)**2*np.cos(2*phidet)+(3+np.exp(2*i*beta))*kappa*(1+rSRM**2)*np.sin(2*phidet))+lambSR*(np.exp(2*i*beta)*rSRM-(1+rSRM**2)/2*(np.cos(2*phidet)+kappa*np.sin(2*phidet)/2)))
    C22=C11
    C12=np.sqrt(1-lambPD)*tSRM**2*(-(np.sin(2*phidet)+kappa*np.sin(phidet)**2)+epsilon*np.sin(phidet)/2*((3+np.exp(2*i*beta))*kappa*np.sin(phidet)+4*np.cos(beta)**2*np.cos(phidet))+lambSR/2*(np.sin(2*phidet)+kappa*np.sin(phidet)**2))
    C21=np.sqrt(1-lambPD)*tSRM**2*((np.sin(2*phidet)-kappa*np.cos(phidet)**2)+epsilon*np.cos(phidet)/2*((3+np.exp(2*i*beta))*kappa*np.cos(phidet)-4*np.cos(beta)**2*np.sin(phidet))+lambSR/2*(-np.sin(2*phidet)+kappa*np.cos(phidet)**2))
    D1sig=np.sqrt(1-lambPD)*(-(1+rSRM*np.exp(2*i*beta))*np.sin(phidet)+epsilon/4*(3+rSRM+2*rSRM*np.exp(4*i*beta)+np.exp(2*i*beta)*(1+5*rSRM))*np.sin(phidet)+lambSR/2*np.exp(2*i*beta)*rSRM*np.sin(phidet))
    D2sig=np.sqrt(1-lambPD)*(-(-1+rSRM*np.exp(2*i*beta))*np.cos(phidet)+epsilon/4*(-3+rSRM+2*rSRM*np.exp(4*i*beta)+np.exp(2*i*beta)*(-1+5*rSRM))*np.cos(phidet)+lambSR/2*np.exp(2*i*beta)*rSRM*np.cos(phidet))
    P11=np.sqrt(1-lambPD)/2*np.sqrt(lambSR)*tSRM*(-2*rSRM*np.exp(2*i*beta)+2*np.cos(2*phidet)+kappa*np.sin(2*phidet))
    P22=P11
    P12=-np.sqrt(1-lambPD)*np.sqrt(lambSR)*tSRM*np.sin(phidet)*(2*np.cos(phidet)+kappa*np.sin(phidet))
    P21=np.sqrt(1-lambPD)*np.sqrt(lambSR)*tSRM*np.cos(phidet)*(2*np.sin(phidet)-kappa*np.cos(phidet))
    Q11=np.sqrt(lambPD)*(np.exp(-2*i*beta)+rSRM**2*np.exp(2*i*beta)-rSRM*(2*np.cos(2*phidet)+kappa*np.sin(2*phidet))+epsilon/2*rSRM*(np.exp(-2*i*beta)*np.cos(2*phidet)+np.exp(2*i*beta)*(-2*rSRM-2*rSRM*np.cos(2*beta)+np.cos(2*phidet)+kappa*np.sin(2*phidet))+2*np.cos(2*phidet)+3*kappa*np.sin(2*phidet))-lambSR/2*rSRM*(2*rSRM*np.exp(2*i*beta)-2*np.cos(2*phidet)-kappa*np.sin(2*phidet)))
    Q22=Q11
    Q12=0
    Q21=0
    N11=np.sqrt(1-lambPD)*np.sqrt(epsilon/2)*tSRM*(kappa*(1+rSRM*np.exp(2*i*beta))*np.sin(phidet)+2*np.cos(beta)*(np.exp(-i*beta)*np.cos(phidet)-rSRM*np.exp(i*beta)*(np.cos(phidet)+kappa*np.sin(phidet))))
    N22=-np.sqrt(1-lambPD)*np.sqrt(2*epsilon)*tSRM*(-np.exp(-i*beta)+rSRM*np.exp(i*beta))*np.cos(beta)*np.cos(phidet)
    N12=-np.sqrt(1-lambPD)*np.sqrt(2*epsilon)*tSRM*(np.exp(-i*beta)+rSRM*np.exp(i*beta))*np.cos(beta)*np.sin(phidet)
    N21=np.sqrt(1-lambPD)*np.sqrt(epsilon/2)*tSRM*(-kappa*(1+rSRM)*np.cos(phidet)+2*np.cos(beta)*(np.exp(-i*beta)+rSRM*np.exp(i*beta))*np.cos(beta)*np.sin(phidet))
    M=0
    
    #power of shot noise
    Ssh=hsql**2/(2*tSRM**2*kappa*np.abs(D1sig*np.sin(xi)+D2sig*np.cos(xi))**2)*(np.abs(C11*np.sin(xi)+C21*np.cos(xi))**2+np.abs(C12*np.sin(xi)+C22*np.cos(xi))**2+np.abs(P11*np.sin(xi)+P21*np.cos(xi))**2+np.abs(P12*np.sin(xi)+P22*np.cos(xi))**2+np.abs(Q11*np.sin(xi)+Q21*np.cos(xi))**2+np.abs(Q12*np.sin(xi)+Q22*np.cos(xi))**2+np.abs(N11*np.sin(xi)+N21*np.cos(xi))**2+np.abs(N12*np.sin(xi)+N22*np.cos(xi))**2+(np.abs(M)*np.sqrt(Sadd))**2)
    
    #These calculations are consistence with the result from Haino.
    
    #strain of shot noise without squeezing
    hsh=np.sqrt(Ssh)

    #squeezing related parameters
    #losses
    lambsq1=0.05   #between squeezer and filter cavity
    lambsq2=30e-6  #inside the filter cavity
    lambsq3=0.05   #between filter cavity and SRM
    if FC:
        Lfilter=float(FC)               #filter cavity length (m)
    else:
        Lfilter=1e10	# dummy
    # for filter cavity parameters, see JGW-T1809537
    gammaRSE=(1+rSRM)/(1-rSRM)*gamma	# RSE linewidth
    I0RSE=(1+rSRM)/(1-rSRM)*I0	# effective input power at BS for RSE (W)
    omegaSQL=sqrt((-gammaRSE**2+sqrt(gammaRSE**4+64*pi*c*I0RSE/(mass*lamb*L**2)))/2)	# SQL reaching frequency for BRSE (rad/s)
    epsilonfilter=4/(2+sqrt(2+2*sqrt(1+(4*omegaSQL*Lfilter/(c*lambsq2))**4)))	# loss parameter
    if epsilonfilter==1.0:
    	epsilonfilter=1.0-1e-6	# to avoid zero division in kappafilter when omegaSQL is too small
    kappafilter=sqrt(2/(2-epsilonfilter)/sqrt(1-epsilonfilter))*omegaSQL/sqrt(2)	# line width of filter cavity (rad/s)
    if not FC:
    	kappafilter=0.0	# 0 for no filter cavity case
    	lambsq2=0.0
    	lambsq3=0.0
    detune=sqrt(1-epsilonfilter)*kappafilter       #detuning of filter cavity (Hz)
    Ttot=kappafilter*4*Lfilter/c #transmittance of filter cavity
    Tfc=Ttot-lambsq2	# filter cavity input mirror transmittance

    squeezelevel=float(SQZ)            # input squeezing in dB
    rsq=log(10)*(squeezelevel/20.0)	# FIXED ON DEC 4, 2018
    sqangle=90*pi/180         #squeezing angle
    
    #covariance matrix of vacuum fluctuations just after the squeezer
    sq11=np.cosh(2*rsq)+np.sinh(2*rsq)*np.cos(2*sqangle)
    sq12=np.sinh(2*rsq)*np.sin(2*sqangle)
    sq22=np.cosh(2*rsq)-np.sinh(2*rsq)*np.cos(2*sqangle)
    
    #quantum noise calculation with squeezing (introduced by Yutaro Enomoto, JGW-T1808243)
    #amplitude reflectivity for sidebands of the filter cavity
    reflp=-np.sqrt(1-Ttot+lambsq2)+((Ttot-lambsq2)*np.sqrt(1-lambsq2)*np.exp(i*2*Lfilter*(detune+omega)/c))/(1-np.sqrt(1-Ttot)*np.exp(i*2*Lfilter*(detune+omega)/c)) #upper
    reflm=-np.sqrt(1-Ttot+lambsq2)+((Ttot-lambsq2)*np.sqrt(1-lambsq2)*np.exp(i*2*Lfilter*(detune-omega)/c))/(1-np.sqrt(1-Ttot)*np.exp(i*2*Lfilter*(detune-omega)/c)) #lower
    
    cov11=(1-lambsq1)*(1-lambsq3)*(np.abs(reflp+np.conj(reflm))**2*sq11/4-np.imag(reflp*reflm)*sq12+np.abs(-reflp+np.conj(reflm))**2*sq22/4)+lambsq1*(1-lambsq3)*(np.abs(reflp)**2+np.abs(reflm)**2)/2+(1-lambsq3)*(2-np.abs(reflp)**2-np.abs(reflm)**2)/2+lambsq3
    cov12=(1-lambsq1)*(1-lambsq3)*(-(np.conj(reflp)+reflm)*(-reflp+np.conj(reflm))*sq11/(4*i)+(np.abs(reflp+np.conj(reflm))**2/4-np.abs(-reflp+np.conj(reflm))**2/4)*sq12-(-np.conj(reflp)+reflm)*(reflp+np.conj(reflm))*sq22/(4*i))+lambsq1*(1-lambsq3)*(np.abs(reflp)**2-np.abs(reflm)**2)/(2*i)+(1-lambsq3)*(-np.abs(reflp)**2+np.abs(reflm)**2)/(2*i)
    cov22=(1-lambsq1)*(1-lambsq3)*(np.abs(-reflp+np.conj(reflm))**2*sq11/4+np.imag(reflp*reflm)*sq12+np.abs(reflp+np.conj(reflm))**2*sq22/4)+lambsq1*(1-lambsq3)*(np.abs(reflp)**2+np.abs(reflm)**2)/2+(1-lambsq3)*(2-np.abs(reflp)**2-np.abs(reflm)**2)/2+lambsq3
    
    #power of shot noise
    Sshsq=hsql**2/(2*tSRM**2*kappa*np.abs(D1sig*np.sin(xi)+D2sig*np.cos(xi))**2)*(np.abs(C11*np.sin(xi)+C21*np.cos(xi))**2*cov11+np.abs(C12*np.sin(xi)+C22*np.cos(xi))**2*cov22+2*np.real(np.conj(C11*np.sin(xi)+C21*np.cos(xi))*(C12*np.sin(xi)+C22*np.cos(xi))*cov12)+np.abs(P11*np.sin(xi)+P21*np.cos(xi))**2+np.abs(P12*np.sin(xi)+P22*np.cos(xi))**2+np.abs(Q11*np.sin(xi)+Q21*np.cos(xi))**2+np.abs(Q12*np.sin(xi)+Q22*np.cos(xi))**2+np.abs(N11*np.sin(xi)+N21*np.cos(xi))**2+np.abs(N12*np.sin(xi)+N22*np.cos(xi))**2+(np.abs(M)*np.sqrt(Sadd))**2)
    
    #strain of shot noise with squeezing
    hshsq=np.sqrt(Sshsq)
    
    if FC:
    	#print "SQL frequency: %.2f Hz" % (omegaSQL/(2*pi))
    	#print "filter cavity input mirror transmissivity: %.2f ppm" % (Tfc*1e6)
    	#print "filter cavity half bandwidth: %.2f Hz" % (kappafilter/(2*pi))
    	#print "filter cavity detuning: %.2f Hz" % (detune/(2*pi))
    	pass
    if SQZ:	# if sqeezing is on
        #print "Maximum detected squeezing: %.2f dB" % (np.max(20*np.log10(hsh/hshsq)))
        #print "Detected squeezing at high freq: %.2f dB" % (20*log10(hsh[-5]/hshsq[-5]))
        hsh=hshsq
    
    #####total noise#####
    htotal=np.sqrt(hseis**2+hsusp**2+hmir**2+hsh**2)
    
    #print hseis[9],hsusp[9],hmir[9],hsh[9]    # FOR DEBUG
    #print hmirsub[9],hmirelaint[9],hmircoa[9] # FOR DEBUG
    
    return {'f':f,'htotal':htotal,'hseis':hseis,'hsusp':hsusp,'hmir':hmir,'hsh':hsh,'hsql':hsql,\
    'phidet':phidet,'xi':xi,'rSRM':rSRM,'Tm':Tm,'I0':I0,'l3':l3,'r1':r1,'r2':r2,'m1':m1,'mass':mass,'w2':w2}
    #'omegaSQL':omegaSQL,'Lfilter':Lfilter,'squeezelevel':squeezelevel,'Tfc':Tfc,'kappafilter':kappafilter,'detune':detune}

def inspiralrange(f,htotal,minsp,*minsp2):
    ## INSPIRAL RANGE ##########
    ## ONLY UPTO ISCO; NOT SUFFICIENT FOR BBH (USE ir_ajith.m instead?)
    ## ALSO, minsp IS DETECTOR FRAME MASS (= SOURCE FRAME MASS * (1+z)).
    SNR=8 #threshold of signal to noise ratio
    averageconst=0.442 #sky average constant given by Kanda-san and see PRD, 47, 2198 (1993).
    iscofactor=1 #upper limit of frequency integration normalized by ISCO frequency
    massstar1=minsp*msolar #mass of star1 (kg)
    try:
        massstar2=minsp2[0]*msolar #mass of star2 (kg)
    except:
        massstar2=massstar1	# if minsp2 not given, same as massstar1
    mtotal=massstar1+massstar2 #total mass (kg)
    mchirp=(massstar1*massstar2)**(3./5)/(massstar1+massstar2)**(1./5) #chirp mass (kg)
    fisco=c**3/(6**(3./2)*pi*G*mtotal) #ISCO frequency
    
    fminset=10 # lower limit of integration
    fminind=np.argmin(np.abs(f-fminset))
    fmaxind=np.argmin(np.abs(f-iscofactor*fisco))
    integfunc=f**(-7./3)/htotal**2 #integrand for inspiral range
    
    sensitivityinteg1=integfunc[fminind:fmaxind]	# index changed from matlab code
    sensitivityinteg2=integfunc[fminind+1:fmaxind+1] # index changed from matlab code
    fbinwidth1=f[fminind:fmaxind] # index changed from matlab code
    fbinwidth2=f[fminind+1:fmaxind+1] # index changed from matlab code
    sensitivityinteg=sum((sensitivityinteg1+sensitivityinteg2)/2*(fbinwidth2-fbinwidth1))
    
    #sensitivityinteg=integ.trapz(integfunc[fminind:fmaxind],f[fminind:fmaxind])	# trapezoidal rule (similar to integration in kagra_sensitivity.m; gives ~1e-6 difference)
    
    # inspiral range (Mpc)
    #see eq 7.182 of M. Maggiore 'Gravitational Waves Volume 1', Oxford University, Press Inc. (2007)
    ir=4*averageconst/SNR*np.sqrt(sensitivityinteg)*np.sqrt(5./96)*pi**(-2./3)*c*(G*mchirp/c**3)**(5./6)/Mpc

    return ir