
import scipy
import numpy as np
from numpy import pi

def coatbrownian(f, ifo):
    """Optical coating Brownian thermal noise
    """

    #coating thermal noise calculations
    #see CQG, 19, 897 (2002)
    #
    # Parameters in kagra_sensitivity.py
    #
    #E0=400e9   #Youngs modulus of mirror (Pa), see Phys. Lett. A 264, 1 (1999)
    #sigma=0.29 #poisson ratio of mirror, see Phys. Lett. A 264, 1 (1999)
    #Qm=1e8     #Q-value of mirror
    #Tm=22
    #
    #####coating parameters
    #
    #number of coating layers
    #Ncoa1=11       #ITM
    #Ncoa2=20       #ETM
    #
    #Youngs modulus (Pa)
    #Ys=73.2e9       #silica,  see PRD 93, 012007 (2016)
    #Yt=140e9        #tantala, see PRD 70, 082003 (2004)
    #
    #poisson ratio
    #sigmas=0.164    #silica,  see PRD 93, 012007 (2016)
    #sigmat=0.23     #tantala, see PRD 70, 082003 (2004)
    #
    #refractive index
    #n1=1.442        #silica,  see PRD 93, 012007 (2016)
    #n2=2.035        #tantala, see CQG 27, 084030 (2010)
    #
    #loss angle
    #phic1=3e-4      #silica coating
    #phic2=5e-4      #tantala coating
    #

    # Parameters in GWINC format
    kb = scipy.constants.k  # Boltzmann constant (J/K)

    # Comment this line out if we use scipy
    kb = 1.38064852e-23     # Boltzmann constant (J/K)

    E0    = ifo.Materials.Substrate.MirrorY
    sigma = ifo.Materials.Substrate.MirrorSigma
    Qm    = ifo.Materials.Substrate.Qvalue
    Tm    = ifo.Materials.Substrate.Temp
    w1    = ifo.Optics.ITM.BeamRadius
    w2    = ifo.Optics.ETM.BeamRadius
    L     = ifo.Infrastructure.Length       # Length of arm cavities [m]
    lamb  = ifo.Laser.Wavelength            # Laser Wavelength [m]
    omega = 2*pi*f

    #####coating parameters
    #
    #number of coating layers
    Ncoa1  = ifo.Optics.ITM.CoatingLayers
    Ncoa2  = ifo.Optics.ETM.CoatingLayers
    #
    #Youngs modulus (Pa)
    Ys     = ifo.Materials.Coating.Ylown
    Yt     = ifo.Materials.Coating.Yhighn
    #
    #poisson ratio
    sigmas = ifo.Materials.Coating.Sigmalown
    sigmat = ifo.Materials.Coating.Sigmahighn
    #
    #refractive index
    n1     = ifo.Materials.Coating.Indexlown
    n2     = ifo.Materials.Coating.Indexhighn
    #
    #thickness of layers (m)
    dcoa1s = (Ncoa1+1)*lamb/(4*n1)  #silica,  ITM
    dcoa1t =  Ncoa1   *lamb/(4*n2)  #tantala, ITM
    dcoa2s = (Ncoa2+1)*lamb/(4*n1)  #silica,  ETM
    dcoa2t =  Ncoa2   *lamb/(4*n2)  #tantala, ITM
    #
    #loss angle
    phic1  = ifo.Materials.Coating.Philown
    phic2  = ifo.Materials.Coating.Phihighn
    #
    CoatImprove = ifo.Materials.Coating.CoatImprove
    if CoatImprove:
        phic1 /= CoatImprove
        phic2 /= CoatImprove

    #thermal noise of silica coating
    Smircoas = 4*kb*Tm/(pi*E0**2*omega) \
        *(phic1*(Ys**2*(1+sigma) **2*(1-2*sigma)**2+ \
                 E0**2*(1+sigmas)**2*(1-2*sigmas))   \
              /(Ys*(1-sigmas**2))*(dcoa1s/w1**2+dcoa2s/w2**2))
    
    #thermal noise of tantala coating
    Smircoat = 4*kb*Tm/(pi*E0**2*omega) \
        *(phic2*(Yt**2*(1+sigma) **2*(1-2*sigma)**2+ \
                 E0**2*(1+sigmat)**2*(1-2*sigmat))   \
              /(Yt*(1-sigmat**2))*(dcoa1t/w1**2+dcoa2t/w2**2))
    
    n = 2 * (Smircoas+Smircoat) * ifo.gwinc.dhdl_sqr

    return n
