
import scipy
import numpy as np
from numpy import pi, sqrt

from gwinc.struct  import Struct

def shotrad_kagra(f, ifo):

    ######quantum noise#####
    #see PRD, 64, 042006 (2001)
    #
    # Parameters in kagra_sensitivity.py
    #
    #c      = 2.99792458e8     #speed of light (m/s)
    #hbar   = 1.054571800e-34  #Dirac constant (J*s)
    #trans  = 0.004            #power transmittivity of ITM
    #rSRM   = 0.92             #amplitude reflectivity of SRM
    #roundtriploss = 100e-6    #round trip loss of arm
    #lambSR = 0.002            #power loss at SRM
    #lambPD = 0.1              #power loss at PD
    #Sadd   = 0*(pi**2/8-1)
    #
    #mass   = 22.8             #mirror mass (kg)
    #L      = 3e3              #arm length (m)
    #lamb   = 1.064e-6         #laser wave length (m)
    #omegal = 2*pi*c/lamb      #laser frequency (rad/s)
    #
    #phidet=pi/180*86.5
    #xi=pi/180*135.1

    # Parameters in GWINC format
    c      = scipy.constants.c             # SOL [m/s]
    hbar   = scipy.constants.hbar          # Plancks Constant [Js]

    # Comment this line out if we use scipy
    hbar   = 1.054571800e-34               # Dirac constant (J*s)

    trans  = ifo.Optics.ITM.Transmittance  # ITM Transmittance [Power]
    tau    = sqrt(ifo.Optics.SRM.Transmittance) # SRM Transmittance [amplitude]
    rSRM   = sqrt(1 - tau**2)                   # SRM Reflectivity  [amplitude]
    roundtriploss = ifo.Optics.Loss*2           # round trip loss of arm
    lambSR = 1 - ifo.Optics.coupling            # power loss at SRM
    lambPD = 1 - ifo.Optics.PhotoDetectorEfficiency  # power loss at PD

    Sadd   = 0*(pi**2/8-1)

    mass   = ifo.Materials.MirrorMass        # Mirror mass [kg]
    L      = ifo.Infrastructure.Length       # Length of arm cavities [m]
    lamb   = ifo.Laser.Wavelength            # Laser Wavelength [m]
    omegal = 2*pi*c/lamb                     # Laser angular frequency [rads/s]

    omega  = 2*pi*f

    I0     = ifo.gwinc.pbs              #power at BS
    i      = 1j

    phidet = pi/2-ifo.Optics.SRM.Tunephase/2      # SRC Detunning

    # Homodyne Readout phase
    xi     = ifo.Optics.get('Quadrature', Struct()).get('dc', None)



    #parameters
    gamma=trans*c/(4*L)       #cavity pole (Hz)
    tSRM=np.sqrt(1-rSRM**2)       #SRM transmittance
    beta=np.arctan(omega/gamma)   #phase shift in the arm cavity
    
    #laser power quantum noise reaches the SQL at the cavity pole
    ISQL=mass*L**2*gamma**4/(4*omegal)
    
    #coefficient showing how much the mirror moves
    kappa=I0/ISQL*2*gamma**4/(omega**2*(gamma**2+omega**2))
    
    #loss parameter of arms defined in KLMTV paper
    epsilon=2*roundtriploss/trans
    
    #quantum noise calculations
    # strain of SQL
    hsql=np.sqrt(8*hbar/(mass*omega**2*L**2))
    
    #components of eq.(5.13) in PRD, 64, 042006 (2001)
    C11=np.sqrt(1-lambPD)*((1+rSRM**2)*(np.cos(2*phidet)+kappa*np.sin(2*phidet)/2)-2*rSRM*np.cos(2*beta)-epsilon/4*(-2*(1+np.exp(2*i*beta))**2*rSRM+4*(1+rSRM**2)*np.cos(beta)**2*np.cos(2*phidet)+(3+np.exp(2*i*beta))*kappa*(1+rSRM**2)*np.sin(2*phidet))+lambSR*(np.exp(2*i*beta)*rSRM-(1+rSRM**2)/2*(np.cos(2*phidet)+kappa*np.sin(2*phidet)/2)))
    C22=C11
    C12=np.sqrt(1-lambPD)*tSRM**2*(-(np.sin(2*phidet)+kappa*np.sin(phidet)**2)+epsilon*np.sin(phidet)/2*((3+np.exp(2*i*beta))*kappa*np.sin(phidet)+4*np.cos(beta)**2*np.cos(phidet))+lambSR/2*(np.sin(2*phidet)+kappa*np.sin(phidet)**2))
    C21=np.sqrt(1-lambPD)*tSRM**2*((np.sin(2*phidet)-kappa*np.cos(phidet)**2)+epsilon*np.cos(phidet)/2*((3+np.exp(2*i*beta))*kappa*np.cos(phidet)-4*np.cos(beta)**2*np.sin(phidet))+lambSR/2*(-np.sin(2*phidet)+kappa*np.cos(phidet)**2))
    D1sig=np.sqrt(1-lambPD)*(-(1+rSRM*np.exp(2*i*beta))*np.sin(phidet)+epsilon/4*(3+rSRM+2*rSRM*np.exp(4*i*beta)+np.exp(2*i*beta)*(1+5*rSRM))*np.sin(phidet)+lambSR/2*np.exp(2*i*beta)*rSRM*np.sin(phidet))
    D2sig=np.sqrt(1-lambPD)*(-(-1+rSRM*np.exp(2*i*beta))*np.cos(phidet)+epsilon/4*(-3+rSRM+2*rSRM*np.exp(4*i*beta)+np.exp(2*i*beta)*(-1+5*rSRM))*np.cos(phidet)+lambSR/2*np.exp(2*i*beta)*rSRM*np.cos(phidet))
    P11=np.sqrt(1-lambPD)/2*np.sqrt(lambSR)*tSRM*(-2*rSRM*np.exp(2*i*beta)+2*np.cos(2*phidet)+kappa*np.sin(2*phidet))
    P22=P11
    P12=-np.sqrt(1-lambPD)*np.sqrt(lambSR)*tSRM*np.sin(phidet)*(2*np.cos(phidet)+kappa*np.sin(phidet))
    P21=np.sqrt(1-lambPD)*np.sqrt(lambSR)*tSRM*np.cos(phidet)*(2*np.sin(phidet)-kappa*np.cos(phidet))
    Q11=np.sqrt(lambPD)*(np.exp(-2*i*beta)+rSRM**2*np.exp(2*i*beta)-rSRM*(2*np.cos(2*phidet)+kappa*np.sin(2*phidet))+epsilon/2*rSRM*(np.exp(-2*i*beta)*np.cos(2*phidet)+np.exp(2*i*beta)*(-2*rSRM-2*rSRM*np.cos(2*beta)+np.cos(2*phidet)+kappa*np.sin(2*phidet))+2*np.cos(2*phidet)+3*kappa*np.sin(2*phidet))-lambSR/2*rSRM*(2*rSRM*np.exp(2*i*beta)-2*np.cos(2*phidet)-kappa*np.sin(2*phidet)))
    Q22=Q11
    Q12=0
    Q21=0
    N11=np.sqrt(1-lambPD)*np.sqrt(epsilon/2)*tSRM*(kappa*(1+rSRM*np.exp(2*i*beta))*np.sin(phidet)+2*np.cos(beta)*(np.exp(-i*beta)*np.cos(phidet)-rSRM*np.exp(i*beta)*(np.cos(phidet)+kappa*np.sin(phidet))))
    N22=-np.sqrt(1-lambPD)*np.sqrt(2*epsilon)*tSRM*(-np.exp(-i*beta)+rSRM*np.exp(i*beta))*np.cos(beta)*np.cos(phidet)
    N12=-np.sqrt(1-lambPD)*np.sqrt(2*epsilon)*tSRM*(np.exp(-i*beta)+rSRM*np.exp(i*beta))*np.cos(beta)*np.sin(phidet)
    N21=np.sqrt(1-lambPD)*np.sqrt(epsilon/2)*tSRM*(-kappa*(1+rSRM)*np.cos(phidet)+2*np.cos(beta)*(np.exp(-i*beta)+rSRM*np.exp(i*beta))*np.cos(beta)*np.sin(phidet))
    M=0
    
    #power of shot noise
    Ssh=hsql**2/(2*tSRM**2*kappa*np.abs(D1sig*np.sin(xi)+D2sig*np.cos(xi))**2)*(np.abs(C11*np.sin(xi)+C21*np.cos(xi))**2+np.abs(C12*np.sin(xi)+C22*np.cos(xi))**2+np.abs(P11*np.sin(xi)+P21*np.cos(xi))**2+np.abs(P12*np.sin(xi)+P22*np.cos(xi))**2+np.abs(Q11*np.sin(xi)+Q21*np.cos(xi))**2+np.abs(Q12*np.sin(xi)+Q22*np.cos(xi))**2+np.abs(N11*np.sin(xi)+N21*np.cos(xi))**2+np.abs(N12*np.sin(xi)+N22*np.cos(xi))**2+(np.abs(M)*np.sqrt(Sadd))**2)
    
    #These calculations are consistence with the result from Haino.
    
    #strain of shot noise without squeezing
    n = Ssh

    #squeezing related parameters
    #
    if 'Squeezer' not in ifo:
        sqzType = 'None'
    else:
        sqzType = ifo.Squeezer.get('Type', 'Freq Independent')

    SQZ = 0
    FC  = 0
    #losses
    lambsq1 = 0
    lambsq2 = 1
    lambsq3 = 0

    if sqzType == 'None':
        pass
    else:
        SQZ = ifo.Squeezer.AmplitudedB /2      # Squeeing in dB
        #losses
        lambsq1 = ifo.Squeezer.lambsq1
        lambsq2 = ifo.Squeezer.lambsq2
        lambsq3 = ifo.Squeezer.lambsq3

    if sqzType == 'Freq Dependent' and 'FilterCavity' in ifo.Squeezer:
        FC = ifo.Squeezer.FilterCavity.L

    if FC:
        Lfilter=FC      # filter cavity length (m)
    else:
        Lfilter=1e10	# dummy

    # for filter cavity parameters, see JGW-T1809537
    gammaRSE=(1+rSRM)/(1-rSRM)*gamma	# RSE linewidth
    I0RSE=(1+rSRM)/(1-rSRM)*I0	# effective input power at BS for RSE (W)

    # SQL reaching frequency for BRSE (rad/s)
    omegaSQL=sqrt((-gammaRSE**2+sqrt(gammaRSE**4 \
                                     +64*pi*c*I0RSE/(mass*lamb*L**2)))/2)
    # loss parameter
    epsilonfilter=4/(2+sqrt(2+2*sqrt(1+(4*omegaSQL*Lfilter/(c*lambsq2))**4)))

    if epsilonfilter==1.0:
	# to avoid zero division in kappafilter when omegaSQL is too small
    	epsilonfilter=1.0-1e-6

    kappafilter=sqrt(2/(2-epsilonfilter)/sqrt(1-epsilonfilter)) \
                *omegaSQL/sqrt(2)  # line width of filter cavity (rad/s)

    if not FC:
    	kappafilter=0.0	# 0 for no filter cavity case
    	lambsq2=0.0
    	lambsq3=0.0

    #detuning of filter cavity (Hz)
    detune=sqrt(1-epsilonfilter)*kappafilter
    Ttot=kappafilter*4*Lfilter/c #transmittance of filter cavity
    Tfc=Ttot-lambsq2	# filter cavity input mirror transmittance

    squeezelevel=float(SQZ)             # input squeezing in dB
    rsq=np.log(10)*(squeezelevel/20.0)	# FIXED ON DEC 4, 2018
    sqangle=90*pi/180                   #squeezing angle
    
    #covariance matrix of vacuum fluctuations just after the squeezer
    sq11=np.cosh(2*rsq)+np.sinh(2*rsq)*np.cos(2*sqangle)
    sq12=np.sinh(2*rsq)*np.sin(2*sqangle)
    sq22=np.cosh(2*rsq)-np.sinh(2*rsq)*np.cos(2*sqangle)
    
    #quantum noise calculation with squeezing
    # (introduced by Yutaro Enomoto, JGW-T1808243)
    #amplitude reflectivity for sidebands of the filter cavity
    reflp=-np.sqrt(1-Ttot+lambsq2)+((Ttot-lambsq2)*np.sqrt(1-lambsq2)*np.exp(i*2*Lfilter*(detune+omega)/c))/(1-np.sqrt(1-Ttot)*np.exp(i*2*Lfilter*(detune+omega)/c)) #upper
    reflm=-np.sqrt(1-Ttot+lambsq2)+((Ttot-lambsq2)*np.sqrt(1-lambsq2)*np.exp(i*2*Lfilter*(detune-omega)/c))/(1-np.sqrt(1-Ttot)*np.exp(i*2*Lfilter*(detune-omega)/c)) #lower
    
    cov11=(1-lambsq1)*(1-lambsq3)*(np.abs(reflp+np.conj(reflm))**2*sq11/4-np.imag(reflp*reflm)*sq12+np.abs(-reflp+np.conj(reflm))**2*sq22/4)+lambsq1*(1-lambsq3)*(np.abs(reflp)**2+np.abs(reflm)**2)/2+(1-lambsq3)*(2-np.abs(reflp)**2-np.abs(reflm)**2)/2+lambsq3
    cov12=(1-lambsq1)*(1-lambsq3)*(-(np.conj(reflp)+reflm)*(-reflp+np.conj(reflm))*sq11/(4*i)+(np.abs(reflp+np.conj(reflm))**2/4-np.abs(-reflp+np.conj(reflm))**2/4)*sq12-(-np.conj(reflp)+reflm)*(reflp+np.conj(reflm))*sq22/(4*i))+lambsq1*(1-lambsq3)*(np.abs(reflp)**2-np.abs(reflm)**2)/(2*i)+(1-lambsq3)*(-np.abs(reflp)**2+np.abs(reflm)**2)/(2*i)
    cov22=(1-lambsq1)*(1-lambsq3)*(np.abs(-reflp+np.conj(reflm))**2*sq11/4+np.imag(reflp*reflm)*sq12+np.abs(reflp+np.conj(reflm))**2*sq22/4)+lambsq1*(1-lambsq3)*(np.abs(reflp)**2+np.abs(reflm)**2)/2+(1-lambsq3)*(2-np.abs(reflp)**2-np.abs(reflm)**2)/2+lambsq3
    
    #power of shot noise
    Sshsq=hsql**2/(2*tSRM**2*kappa*np.abs(D1sig*np.sin(xi)+D2sig*np.cos(xi))**2)*(np.abs(C11*np.sin(xi)+C21*np.cos(xi))**2*cov11+np.abs(C12*np.sin(xi)+C22*np.cos(xi))**2*cov22+2*np.real(np.conj(C11*np.sin(xi)+C21*np.cos(xi))*(C12*np.sin(xi)+C22*np.cos(xi))*cov12)+np.abs(P11*np.sin(xi)+P21*np.cos(xi))**2+np.abs(P12*np.sin(xi)+P22*np.cos(xi))**2+np.abs(Q11*np.sin(xi)+Q21*np.cos(xi))**2+np.abs(Q12*np.sin(xi)+Q22*np.cos(xi))**2+np.abs(N11*np.sin(xi)+N21*np.cos(xi))**2+np.abs(N12*np.sin(xi)+N22*np.cos(xi))**2+(np.abs(M)*np.sqrt(Sadd))**2)
    
    #strain of shot noise with squeezing
    #hshsq=np.sqrt(Sshsq)
    
    if FC:
    	#print "SQL frequency: %.2f Hz" % (omegaSQL/(2*pi))
    	#print "filter cavity input mirror transmissivity: %.2f ppm" % (Tfc*1e6)
    	#print "filter cavity half bandwidth: %.2f Hz" % (kappafilter/(2*pi))
    	#print "filter cavity detuning: %.2f Hz" % (detune/(2*pi))
    	pass
    if SQZ:	# if sqeezing is on
        #print "Maximum detected squeezing: %.2f dB" % (np.max(20*np.log10(hsh/hshsq)))
        #print "Detected squeezing at high freq: %.2f dB" % (20*log10(hsh[-5]/hshsq[-5]))
        n = Sshsq #hsh=hshsq

    return n * ifo.gwinc.sinc_sqr

def shotrad(f, ifo):
    """Quantum noise
    """

    return shotrad_kagra(f, ifo)
