
import scipy
import numpy as np
from numpy import pi

def subtherm(f, ifo):
    """Noise from thermoelastic fluctuations in mirror
    """

    ######mirror thermal noise#####
    #thermoelastic noise calculations
    #see PRD, 82, 127101 (2010)
    #
    # Parameters in kagra_sensitivity.py
    #
    #sigma=0.29 #poisson ratio of mirror, see Phys. Lett. A 264, 1 (1999)
    #Tm=22
    #rhom=4.0e3      #density of mirror (kg/m**3) #sapphire

    # Parameters in GWINC format
    kb = scipy.constants.k  # Boltzmann constant (J/K)

    # Comment this line out if we use scipy
    kb = 1.38064852e-23     # Boltzmann constant (J/K)

    rhom  = ifo.Materials.Substrate.MassDensity
    w1    = ifo.Optics.ITM.BeamRadius
    w2    = ifo.Optics.ETM.BeamRadius
    Tm    = ifo.Materials.Substrate.Temp
    sigma = ifo.Materials.Substrate.MirrorSigma
    L     = ifo.Infrastructure.Length       # Length of arm cavities [m]
    omega = 2*pi*f

    #From Uchiyama-san. Origninally from Touloukian and Ho,
    # Thermophysical Properties of Matter: The TRPC Data Series,
    #  Plenum Press (1970).
    #see PRL 108, 141101 (2012).

    #thermal conductivity
    kappa20 = (15700./15880)*((5*Tm**  2.75)**(-4./5)
                     +(10**10.25*Tm**(-3.8))**(-4./5))**(-5./4)
    Cth20   = (0.69/0.80)*Tm**3.14/3.8/rhom   #specific heat per unit mass
    alpha20 = (5.6/5.496)*Tm**2.99/10**12.15  #thermal linear expansion

    #thermoelastic noise calculations
    #see PRD, 82, 127101 (2010)
    #normalized frequency
    omegac1 = omega*Cth20*rhom*w1**2/(2*kappa20)
    omegac2 = omega*Cth20*rhom*w2**2/(2*kappa20)
    
    # SH: rhom is actually cancelled

    #calculation of Jhyper
    #It is confirmed by K. Komori that this integration is
    #  the same as the analytical calculation using error function.
    resolution = 0.01
    up  = 10    #upper limit of integration instead of infinity
    bin = int(np.floor(up/resolution))
    u   = np.linspace(0,up,bin+1)
    
    # for-loop omitted and simplified from matlab code for speed up
    #  (no change in result)
    uu,omec1 = np.meshgrid(u, omegac1)
    uu,omec2 = np.meshgrid(u, omegac2)
    
    integ1 = u**2*np.exp(-u**2/2)*(1-u/np.sqrt(2) \
                      *np.sqrt((u**2+np.sqrt(u**4+omec1**2))/(u**4+omec1**2)))
    integ2 = u**2*np.exp(-u**2/2)*(1-u/np.sqrt(2) \
                      *np.sqrt((u**2+np.sqrt(u**4+omec2**2))/(u**4+omec2**2)))
    
    # nulling changed from matlab code
    integral1 = np.sum(integ1[:,1:]+integ1[:,:-1],axis=1)/2*resolution
    integral2 = np.sum(integ2[:,1:]+integ2[:,:-1],axis=1)/2*resolution
    
    Jhyper1int = np.sqrt(2/pi)*integral1/omegac1**2
    Jhyper2int = np.sqrt(2/pi)*integral2/omegac2**2
    
    #strain of thermoelastic noise for 2 ITMs and ETMs
    hmirelaint = 4*kb*Tm**2*alpha20**2*(1+sigma)**2 \
                 /(np.sqrt(pi)*kappa20)*(w1*Jhyper1int+w2*Jhyper2int)

    # account for 2 ITM and 2 ETM, and convert to strain whith 1/L^2
    n = 2 * hmirelaint /L**2 #* ifo.gwinc.dhdl_sqr

    return n


def subbrownian(f, ifo):
    """Strain noise from the Brownian thermal noise 
       due to substrate mechanical loss
    """

    ######mirror thermal noise#####
    #strain of substrate thermal noise for 2 ITMs and ETMs
    #see CQG, 29, 124007 (2012)
    #
    # Parameters in kagra_sensitivity.py
    #
    #E0=400e9   #Youngs modulus of mirror (Pa), see Phys. Lett. A 264, 1 (1999)
    #sigma=0.29 #poisson ratio of mirror, see Phys. Lett. A 264, 1 (1999)
    #Qm=1e8     #Q-value of mirror
    #Tm=22

    # Parameters in GWINC format
    kb = scipy.constants.k  # Boltzmann constant (J/K)

    # Comment this line out if we use scipy
    kb = 1.38064852e-23     # Boltzmann constant (J/K)

    E0    = ifo.Materials.Substrate.MirrorY
    sigma = ifo.Materials.Substrate.MirrorSigma
    Qm    = ifo.Materials.Substrate.Qvalue
    Tm    = ifo.Materials.Substrate.Temp
    w1    = ifo.Optics.ITM.BeamRadius
    w2    = ifo.Optics.ETM.BeamRadius
    L     = ifo.Infrastructure.Length       # Length of arm cavities [m]
    omega = 2*pi*f

    hmirsub = 4*kb*Tm*(1-sigma**2)/(np.sqrt(pi)*E0*Qm*omega)*(1/w1+1/w2)

    # account for 2 ITM and 2 ETM, and convert to strain whith 1/L^2
    n = 2 * hmirsub * ifo.gwinc.dhdl_sqr

    return n
